@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2014, Adrian Sampson.
+# Copyright 2016, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -32,8 +33,11 @@ Internally ``MediaFile`` uses ``MediaField`` descriptors to access the
 data from the tags. In turn ``MediaField`` uses a number of
 ``StorageStyle`` strategies to handle format specific logic.
 """
+from __future__ import division, absolute_import, print_function
+
 import mutagen
 import mutagen.mp3
+import mutagen.id3
 import mutagen.oggopus
 import mutagen.oggvorbis
 import mutagen.mp4
@@ -41,18 +45,20 @@ import mutagen.flac
 import mutagen.monkeysaudio
 import mutagen.asf
 import mutagen.aiff
+import codecs
 import datetime
 import re
 import base64
+import binascii
 import math
 import struct
 import imghdr
 import os
-import logging
 import traceback
 import enum
 
-from beets.util import displayable_path
+from beets import logging
+from beets.util import displayable_path, syspath, as_string
 
 
 __all__ = ['UnreadableFileError', 'FileTypeError', 'MediaFile']
@@ -141,7 +147,7 @@ def _safe_cast(out_type, val):
             return False
 
     elif out_type == unicode:
-        if isinstance(val, str):
+        if isinstance(val, bytes):
             return val.decode('utf8', 'ignore')
         elif isinstance(val, unicode):
             return val
@@ -152,9 +158,12 @@ def _safe_cast(out_type, val):
         if isinstance(val, int) or isinstance(val, float):
             return float(val)
         else:
-            if not isinstance(val, basestring):
+            if isinstance(val, bytes):
+                val = val.decode('utf8', 'ignore')
+            else:
                 val = unicode(val)
-            match = re.match(r'[\+-]?[0-9\.]+', val.strip())
+            match = re.match(r'[\+-]?([0-9]+\.?[0-9]*|[0-9]*\.[0-9]+)',
+                             val.strip())
             if match:
                 val = match.group(0)
                 if val:
@@ -176,15 +185,15 @@ def _unpack_asf_image(data):
     of exceptions (out-of-bounds, etc.). We should clean this up
     sometime so that the failure modes are well-defined.
     """
-    type, size = struct.unpack_from("<bi", data)
+    type, size = struct.unpack_from('<bi', data)
     pos = 5
-    mime = ""
-    while data[pos:pos + 2] != "\x00\x00":
+    mime = b''
+    while data[pos:pos + 2] != b'\x00\x00':
         mime += data[pos:pos + 2]
         pos += 2
     pos += 2
-    description = ""
-    while data[pos:pos + 2] != "\x00\x00":
+    description = b''
+    while data[pos:pos + 2] != b'\x00\x00':
         description += data[pos:pos + 2]
         pos += 2
     pos += 2
@@ -196,9 +205,9 @@ def _unpack_asf_image(data):
 def _pack_asf_image(mime, data, type=3, description=""):
     """Pack image data for a WM/Picture tag.
     """
-    tag_data = struct.pack("<bi", type, len(data))
-    tag_data += mime.encode("utf-16-le") + "\x00\x00"
-    tag_data += description.encode("utf-16-le") + "\x00\x00"
+    tag_data = struct.pack('<bi', type, len(data))
+    tag_data += mime.encode("utf-16-le") + b'\x00\x00'
+    tag_data += description.encode("utf-16-le") + b'\x00\x00'
     tag_data += data
     return tag_data
 
@@ -206,15 +215,20 @@ def _pack_asf_image(mime, data, type=3, description=""):
 # iTunes Sound Check encoding.
 
 def _sc_decode(soundcheck):
-    """Convert a Sound Check string value to a (gain, peak) tuple as
+    """Convert a Sound Check bytestring value to a (gain, peak) tuple as
     used by ReplayGain.
     """
+    # We decode binary data. If one of the formats gives us a text
+    # string, interpret it as UTF-8.
+    if isinstance(soundcheck, unicode):
+        soundcheck = soundcheck.encode('utf8')
+
     # SoundCheck tags consist of 10 numbers, each represented by 8
     # characters of ASCII hex preceded by a space.
     try:
-        soundcheck = soundcheck.replace(' ', '').decode('hex')
+        soundcheck = codecs.decode(soundcheck.replace(b' ', b''), 'hex')
         soundcheck = struct.unpack('!iiiiiiiiii', soundcheck)
-    except (struct.error, TypeError, UnicodeEncodeError):
+    except (struct.error, TypeError, binascii.Error):
         # SoundCheck isn't in the format we expect, so return default
         # values.
         return 0.0, 0.0
@@ -267,10 +281,23 @@ def _sc_encode(gain, peak):
 
 # Cover art and other images.
 
+def _wider_test_jpeg(data):
+    """Test for a jpeg file following the UNIX file implementation which
+    uses the magic bytes rather than just looking for the bytes that
+    represent 'JFIF' or 'EXIF' at a fixed position.
+    """
+    if data[:2] == b'\xff\xd8':
+        return 'jpeg'
+
+
 def _image_mime_type(data):
     """Return the MIME type of the image data (a bytestring).
     """
-    kind = imghdr.what(None, h=data)
+    # This checks for a jpeg file with only the magic bytes (unrecognized by
+    # imghdr.what). imghdr.what returns none for that type of file, so
+    # _wider_test_jpeg is run in that case. It still returns None if it didn't
+    # match such a jpeg file.
+    kind = imghdr.what(None, h=data) or _wider_test_jpeg(data)
     if kind in ['gif', 'jpeg', 'png', 'tiff', 'bmp']:
         return 'image/{0}'.format(kind)
     elif kind == 'pgm':
@@ -312,12 +339,12 @@ class ImageType(enum.Enum):
 
 
 class Image(object):
-    """Strucuture representing image data and metadata that can be
+    """Structure representing image data and metadata that can be
     stored and retrieved from tags.
 
     The structure has four properties.
     * ``data``  The binary data of the image
-    * ``desc``  An optional descritpion of the image
+    * ``desc``  An optional description of the image
     * ``type``  An instance of `ImageType` indicating the kind of image
     * ``mime_type`` Read-only property that contains the mime type of
                     the binary data
@@ -326,7 +353,11 @@ class Image(object):
         self.data = data
         self.desc = desc
         if isinstance(type, int):
-            type = list(ImageType)[type]
+            try:
+                type = list(ImageType)[type]
+            except IndexError:
+                log.debug(u"ignoring unknown image type index {0}", type)
+                type = ImageType.other
         self.type = type
 
     @property
@@ -395,8 +426,9 @@ class StorageStyle(object):
         self.float_places = float_places
 
         # Convert suffix to correct string type.
-        if self.suffix and self.as_type is unicode:
-            self.suffix = self.as_type(self.suffix)
+        if self.suffix and self.as_type is unicode \
+           and not isinstance(self.suffix, unicode):
+            self.suffix = self.suffix.decode('utf8')
 
     # Getter.
 
@@ -447,7 +479,7 @@ class StorageStyle(object):
             if isinstance(value, bool):
                 # Store bools as 1/0 instead of True/False.
                 value = unicode(int(bool(value)))
-            elif isinstance(value, str):
+            elif isinstance(value, bytes):
                 value = value.decode('utf8', 'ignore')
             else:
                 value = unicode(value)
@@ -622,13 +654,13 @@ class MP4BoolStorageStyle(MP4StorageStyle):
             return None
 
     def get_list(self, mutagen_file):
-        raise NotImplementedError('MP4 bool storage does not support lists')
+        raise NotImplementedError(u'MP4 bool storage does not support lists')
 
     def set(self, mutagen_file, value):
         mutagen_file[self.key] = value
 
     def set_list(self, mutagen_file, values):
-        raise NotImplementedError('MP4 bool storage does not support lists')
+        raise NotImplementedError(u'MP4 bool storage does not support lists')
 
 
 class MP4ImageStorageStyle(MP4ListStorageStyle):
@@ -646,7 +678,7 @@ class MP4ImageStorageStyle(MP4ListStorageStyle):
         elif image.mime_type == 'image/jpeg':
             kind = mutagen.mp4.MP4Cover.FORMAT_JPEG
         else:
-            raise ValueError('MP4 files only supports PNG and JPEG images')
+            raise ValueError(u'MP4 files only supports PNG and JPEG images')
         return mutagen.mp4.MP4Cover(image.data, kind)
 
 
@@ -725,19 +757,20 @@ class MP3DescStorageStyle(MP3StorageStyle):
         if self.key != 'USLT':
             value = [value]
 
-        # try modifying in place
+        # Try modifying in place.
         found = False
         for frame in frames:
             if frame.desc.lower() == self.description.lower():
                 frame.text = value
+                frame.encoding = mutagen.id3.Encoding.UTF8
                 found = True
 
-        # need to make a new frame?
+        # Try creating a new frame.
         if not found:
             frame = mutagen.id3.Frames[self.key](
-                desc=str(self.description),
+                desc=bytes(self.description),
                 text=value,
-                encoding=3
+                encoding=mutagen.id3.Encoding.UTF8,
             )
             if self.id3_lang:
                 frame.lang = self.id3_lang
@@ -804,11 +837,11 @@ class MP3ImageStorageStyle(ListStorageStyle, MP3StorageStyle):
 
     The `get_list` method inherited from ``ListStorageStyle`` returns a
     list of ``Image``s. Similarly, the `set_list` method accepts a
-    list of ``Image``s as its ``values`` arguemnt.
+    list of ``Image``s as its ``values`` argument.
     """
     def __init__(self):
         super(MP3ImageStorageStyle, self).__init__(key='APIC')
-        self.as_type = str
+        self.as_type = bytes
 
     def deserialize(self, apic_frame):
         """Convert APIC frame into Image."""
@@ -821,6 +854,9 @@ class MP3ImageStorageStyle(ListStorageStyle, MP3StorageStyle):
     def store(self, mutagen_file, frames):
         mutagen_file.tags.setall(self.key, frames)
 
+    def delete(self, mutagen_file):
+        mutagen_file.tags.delall(self.key)
+
     def serialize(self, image):
         """Return an APIC frame populated with data from ``image``.
         """
@@ -868,13 +904,13 @@ class VorbisImageStorageStyle(ListStorageStyle):
     base64-encoded. Values are `Image` objects.
     """
     formats = ['OggOpus', 'OggTheora', 'OggSpeex', 'OggVorbis',
-               'OggFlac', 'APEv2File', 'WavPack', 'Musepack', 'MonkeysAudio']
+               'OggFlac']
 
     def __init__(self):
         super(VorbisImageStorageStyle, self).__init__(
             key='metadata_block_picture'
         )
-        self.as_type = str
+        self.as_type = bytes
 
     def fetch(self, mutagen_file):
         images = []
@@ -950,6 +986,74 @@ class FlacImageStorageStyle(ListStorageStyle):
         mutagen_file.clear_pictures()
 
 
+class APEv2ImageStorageStyle(ListStorageStyle):
+    """Store images in APEv2 tags. Values are `Image` objects.
+    """
+    formats = ['APEv2File', 'WavPack', 'Musepack', 'MonkeysAudio', 'OptimFROG']
+
+    TAG_NAMES = {
+        ImageType.other: 'Cover Art (other)',
+        ImageType.icon: 'Cover Art (icon)',
+        ImageType.other_icon: 'Cover Art (other icon)',
+        ImageType.front: 'Cover Art (front)',
+        ImageType.back: 'Cover Art (back)',
+        ImageType.leaflet: 'Cover Art (leaflet)',
+        ImageType.media: 'Cover Art (media)',
+        ImageType.lead_artist: 'Cover Art (lead)',
+        ImageType.artist: 'Cover Art (artist)',
+        ImageType.conductor: 'Cover Art (conductor)',
+        ImageType.group: 'Cover Art (band)',
+        ImageType.composer: 'Cover Art (composer)',
+        ImageType.lyricist: 'Cover Art (lyricist)',
+        ImageType.recording_location: 'Cover Art (studio)',
+        ImageType.recording_session: 'Cover Art (recording)',
+        ImageType.performance: 'Cover Art (performance)',
+        ImageType.screen_capture: 'Cover Art (movie scene)',
+        ImageType.fish: 'Cover Art (colored fish)',
+        ImageType.illustration: 'Cover Art (illustration)',
+        ImageType.artist_logo: 'Cover Art (band logo)',
+        ImageType.publisher_logo: 'Cover Art (publisher logo)',
+    }
+
+    def __init__(self):
+        super(APEv2ImageStorageStyle, self).__init__(key='')
+
+    def fetch(self, mutagen_file):
+        images = []
+        for cover_type, cover_tag in self.TAG_NAMES.items():
+            try:
+                frame = mutagen_file[cover_tag]
+                text_delimiter_index = frame.value.find(b'\x00')
+                comment = frame.value[0:text_delimiter_index] \
+                    if text_delimiter_index > 0 else None
+                image_data = frame.value[text_delimiter_index + 1:]
+                images.append(Image(data=image_data, type=cover_type,
+                                    desc=comment))
+            except KeyError:
+                pass
+
+        return images
+
+    def set_list(self, mutagen_file, values):
+        self.delete(mutagen_file)
+
+        for image in values:
+            image_type = image.type or ImageType.other
+            comment = image.desc or ''
+            image_data = comment.encode('utf8') + b'\x00' + image.data
+            cover_tag = self.TAG_NAMES[image_type]
+            mutagen_file[cover_tag] = image_data
+
+    def delete(self, mutagen_file):
+        """Remove all images from the file.
+        """
+        for cover_tag in self.TAG_NAMES.values():
+            try:
+                del mutagen_file[cover_tag]
+            except KeyError:
+                pass
+
+
 # MediaField is a descriptor that represents a single logical field. It
 # aggregates several StorageStyles describing how to access the data for
 # each file type.
@@ -1088,11 +1192,11 @@ class DateField(MediaField):
         year, month, and day number. Each number is either an integer or
         None.
         """
-        # Get the underlying data and split on hyphens.
+        # Get the underlying data and split on hyphens and slashes.
         datestring = super(DateField, self).__get__(mediafile, None)
         if isinstance(datestring, basestring):
             datestring = re.sub(r'[Tt ].*$', '', unicode(datestring))
-            items = unicode(datestring).split('-')
+            items = re.split('[-/]', unicode(datestring))
         else:
             items = []
 
@@ -1167,18 +1271,31 @@ class DateItemField(MediaField):
 
 class CoverArtField(MediaField):
     """A descriptor that provides access to the *raw image data* for the
-    first image on a file. This is used for backwards compatibility: the
+    cover image on a file. This is used for backwards compatibility: the
     full `ImageListField` provides richer `Image` objects.
+
+    When there are multiple images we try to pick the most likely to be a front
+    cover.
     """
     def __init__(self):
         pass
 
     def __get__(self, mediafile, _):
-        try:
-            return mediafile.images[0].data
-        except IndexError:
+        candidates = mediafile.images
+        if candidates:
+            return self.guess_cover_image(candidates).data
+        else:
             return None
 
+    @staticmethod
+    def guess_cover_image(candidates):
+        if len(candidates) == 1:
+            return candidates[0]
+        try:
+            return next(c for c in candidates if c.type == ImageType.front)
+        except StopIteration:
+            return candidates[0]
+
     def __set__(self, mediafile, data):
         if data:
             mediafile.images = [Image(data=data)]
@@ -1189,7 +1306,7 @@ class CoverArtField(MediaField):
         delattr(mediafile, 'images')
 
 
-class ImageListField(MediaField):
+class ImageListField(ListMediaField):
     """Descriptor to access the list of images embedded in tags.
 
     The getter returns a list of `Image` instances obtained from
@@ -1206,18 +1323,10 @@ class ImageListField(MediaField):
             ASFImageStorageStyle(),
             VorbisImageStorageStyle(),
             FlacImageStorageStyle(),
+            APEv2ImageStorageStyle(),
+            out_type=Image,
         )
 
-    def __get__(self, mediafile, _):
-        images = []
-        for style in self.styles(mediafile.mgfile):
-            images.extend(style.get_list(mediafile.mgfile))
-        return images
-
-    def __set__(self, mediafile, images):
-        for style in self.styles(mediafile.mgfile):
-            style.set_list(mediafile.mgfile, images)
-
 
 # MediaFile is a collection of fields.
 
@@ -1232,6 +1341,7 @@ class MediaFile(object):
         By default, MP3 files are saved with ID3v2.4 tags. You can use
         the older ID3v2.3 standard by specifying the `id3v23` option.
         """
+        path = syspath(path)
         self.path = path
 
         unreadable_exc = (
@@ -1250,7 +1360,7 @@ class MediaFile(object):
         try:
             self.mgfile = mutagen.File(path)
         except unreadable_exc as exc:
-            log.debug(u'header parsing failed: {0}'.format(unicode(exc)))
+            log.debug(u'header parsing failed: {0}', unicode(exc))
             raise UnreadableFileError(path)
         except IOError as exc:
             if type(exc) == IOError:
@@ -1258,12 +1368,12 @@ class MediaFile(object):
                 # anywhere else.
                 raise
             else:
-                log.debug(traceback.format_exc())
+                log.debug(u'{}', traceback.format_exc())
                 raise MutagenError(path, exc)
         except Exception as exc:
             # Isolate bugs in Mutagen.
-            log.debug(traceback.format_exc())
-            log.error(u'uncaught Mutagen exception in open: {0}'.format(exc))
+            log.debug(u'{}', traceback.format_exc())
+            log.error(u'uncaught Mutagen exception in open: {0}', exc)
             raise MutagenError(path, exc)
 
         if self.mgfile is None:
@@ -1271,15 +1381,21 @@ class MediaFile(object):
             raise FileTypeError(path)
         elif (type(self.mgfile).__name__ == 'M4A' or
               type(self.mgfile).__name__ == 'MP4'):
-            # This hack differentiates AAC and ALAC until we find a more
-            # deterministic approach. Mutagen only sets the sample rate
-            # for AAC files. See:
-            # https://github.com/sampsyo/beets/pull/295
-            if hasattr(self.mgfile.info, 'sample_rate') and \
-               self.mgfile.info.sample_rate > 0:
-                self.type = 'aac'
+            info = self.mgfile.info
+            if hasattr(info, 'codec'):
+                if info.codec and info.codec.startswith('alac'):
+                    self.type = 'alac'
+                else:
+                    self.type = 'aac'
             else:
-                self.type = 'alac'
+                # This hack differentiates AAC and ALAC on versions of
+                # Mutagen < 1.26. Once Mutagen > 1.26 is out and
+                # required by beets, we can remove this.
+                if hasattr(self.mgfile.info, 'bitrate') and \
+                   self.mgfile.info.bitrate > 0:
+                    self.type = 'aac'
+                else:
+                    self.type = 'alac'
         elif (type(self.mgfile).__name__ == 'ID3' or
               type(self.mgfile).__name__ == 'MP3'):
             self.type = 'mp3'
@@ -1329,8 +1445,8 @@ class MediaFile(object):
             # Propagate these through: they don't represent Mutagen bugs.
             raise
         except Exception as exc:
-            log.debug(traceback.format_exc())
-            log.error(u'uncaught Mutagen exception in save: {0}'.format(exc))
+            log.debug(u'{}', traceback.format_exc())
+            log.error(u'uncaught Mutagen exception in save: {0}', exc)
             raise MutagenError(self.path, exc)
 
     def delete(self):
@@ -1354,7 +1470,35 @@ class MediaFile(object):
         """
         for property, descriptor in cls.__dict__.items():
             if isinstance(descriptor, MediaField):
-                yield property
+                yield as_string(property)
+
+    @classmethod
+    def _field_sort_name(cls, name):
+        """Get a sort key for a field name that determines the order
+        fields should be written in.
+
+        Fields names are kept unchanged, unless they are instances of
+        :class:`DateItemField`, in which case `year`, `month`, and `day`
+        are replaced by `date0`, `date1`, and `date2`, respectively, to
+        make them appear in that order.
+        """
+        if isinstance(cls.__dict__[name], DateItemField):
+            name = re.sub('year',  'date0', name)
+            name = re.sub('month', 'date1', name)
+            name = re.sub('day',   'date2', name)
+        return name
+
+    @classmethod
+    def sorted_fields(cls):
+        """Get the names of all writable metadata fields, sorted in the
+        order that they should be written.
+
+        This is a lexicographic order, except for instances of
+        :class:`DateItemField`, which are sorted in year-month-day
+        order.
+        """
+        for property in sorted(cls.fields(), key=cls._field_sort_name):
+            yield property
 
     @classmethod
     def readable_fields(cls):
@@ -1392,7 +1536,7 @@ class MediaFile(object):
         the `MediaFile`. If a key has the value `None`, the
         corresponding property is deleted from the `MediaFile`.
         """
-        for field in self.fields():
+        for field in self.sorted_fields():
             if field in dict:
                 if dict[field] is None:
                     delattr(self, field)
@@ -1403,25 +1547,25 @@ class MediaFile(object):
 
     title = MediaField(
         MP3StorageStyle('TIT2'),
-        MP4StorageStyle("\xa9nam"),
+        MP4StorageStyle('\xa9nam'),
         StorageStyle('TITLE'),
         ASFStorageStyle('Title'),
     )
     artist = MediaField(
         MP3StorageStyle('TPE1'),
-        MP4StorageStyle("\xa9ART"),
+        MP4StorageStyle('\xa9ART'),
         StorageStyle('ARTIST'),
         ASFStorageStyle('Author'),
     )
     album = MediaField(
         MP3StorageStyle('TALB'),
-        MP4StorageStyle("\xa9alb"),
+        MP4StorageStyle('\xa9alb'),
         StorageStyle('ALBUM'),
         ASFStorageStyle('WM/AlbumTitle'),
     )
     genres = ListMediaField(
         MP3ListStorageStyle('TCON'),
-        MP4ListStorageStyle("\xa9gen"),
+        MP4ListStorageStyle('\xa9gen'),
         ListStorageStyle('GENRE'),
         ASFStorageStyle('WM/Genre'),
     )
@@ -1429,13 +1573,13 @@ class MediaFile(object):
 
     composer = MediaField(
         MP3StorageStyle('TCOM'),
-        MP4StorageStyle("\xa9wrt"),
+        MP4StorageStyle('\xa9wrt'),
         StorageStyle('COMPOSER'),
         ASFStorageStyle('WM/Composer'),
     )
     grouping = MediaField(
         MP3StorageStyle('TIT1'),
-        MP4StorageStyle("\xa9grp"),
+        MP4StorageStyle('\xa9grp'),
         StorageStyle('GROUPING'),
         ASFStorageStyle('WM/ContentGroupDescription'),
     )
@@ -1475,16 +1619,17 @@ class MediaFile(object):
     )
     lyrics = MediaField(
         MP3DescStorageStyle(key='USLT'),
-        MP4StorageStyle("\xa9lyr"),
+        MP4StorageStyle('\xa9lyr'),
         StorageStyle('LYRICS'),
         ASFStorageStyle('WM/Lyrics'),
     )
     comments = MediaField(
         MP3DescStorageStyle(key='COMM'),
-        MP4StorageStyle("\xa9cmt"),
+        MP4StorageStyle('\xa9cmt'),
         StorageStyle('DESCRIPTION'),
         StorageStyle('COMMENT'),
         ASFStorageStyle('WM/Comments'),
+        ASFStorageStyle('Description')
     )
     bpm = MediaField(
         MP3StorageStyle('TBPM'),
@@ -1523,76 +1668,76 @@ class MediaFile(object):
     )
     artist_sort = MediaField(
         MP3StorageStyle('TSOP'),
-        MP4StorageStyle("soar"),
+        MP4StorageStyle('soar'),
         StorageStyle('ARTISTSORT'),
         ASFStorageStyle('WM/ArtistSortOrder'),
     )
     albumartist_sort = MediaField(
         MP3DescStorageStyle(u'ALBUMARTISTSORT'),
-        MP4StorageStyle("soaa"),
+        MP4StorageStyle('soaa'),
         StorageStyle('ALBUMARTISTSORT'),
         ASFStorageStyle('WM/AlbumArtistSortOrder'),
     )
     asin = MediaField(
         MP3DescStorageStyle(u'ASIN'),
-        MP4StorageStyle("----:com.apple.iTunes:ASIN"),
+        MP4StorageStyle('----:com.apple.iTunes:ASIN'),
         StorageStyle('ASIN'),
         ASFStorageStyle('MusicBrainz/ASIN'),
     )
     catalognum = MediaField(
         MP3DescStorageStyle(u'CATALOGNUMBER'),
-        MP4StorageStyle("----:com.apple.iTunes:CATALOGNUMBER"),
+        MP4StorageStyle('----:com.apple.iTunes:CATALOGNUMBER'),
         StorageStyle('CATALOGNUMBER'),
         ASFStorageStyle('WM/CatalogNo'),
     )
     disctitle = MediaField(
         MP3StorageStyle('TSST'),
-        MP4StorageStyle("----:com.apple.iTunes:DISCSUBTITLE"),
+        MP4StorageStyle('----:com.apple.iTunes:DISCSUBTITLE'),
         StorageStyle('DISCSUBTITLE'),
         ASFStorageStyle('WM/SetSubTitle'),
     )
     encoder = MediaField(
         MP3StorageStyle('TENC'),
-        MP4StorageStyle("\xa9too"),
+        MP4StorageStyle('\xa9too'),
         StorageStyle('ENCODEDBY'),
         StorageStyle('ENCODER'),
         ASFStorageStyle('WM/EncodedBy'),
     )
     script = MediaField(
         MP3DescStorageStyle(u'Script'),
-        MP4StorageStyle("----:com.apple.iTunes:SCRIPT"),
+        MP4StorageStyle('----:com.apple.iTunes:SCRIPT'),
         StorageStyle('SCRIPT'),
         ASFStorageStyle('WM/Script'),
     )
     language = MediaField(
         MP3StorageStyle('TLAN'),
-        MP4StorageStyle("----:com.apple.iTunes:LANGUAGE"),
+        MP4StorageStyle('----:com.apple.iTunes:LANGUAGE'),
         StorageStyle('LANGUAGE'),
         ASFStorageStyle('WM/Language'),
     )
     country = MediaField(
         MP3DescStorageStyle('MusicBrainz Album Release Country'),
-        MP4StorageStyle("----:com.apple.iTunes:MusicBrainz "
-                        "Album Release Country"),
+        MP4StorageStyle('----:com.apple.iTunes:MusicBrainz '
+                        'Album Release Country'),
         StorageStyle('RELEASECOUNTRY'),
         ASFStorageStyle('MusicBrainz/Album Release Country'),
     )
     albumstatus = MediaField(
         MP3DescStorageStyle(u'MusicBrainz Album Status'),
-        MP4StorageStyle("----:com.apple.iTunes:MusicBrainz Album Status"),
+        MP4StorageStyle('----:com.apple.iTunes:MusicBrainz Album Status'),
         StorageStyle('MUSICBRAINZ_ALBUMSTATUS'),
         ASFStorageStyle('MusicBrainz/Album Status'),
     )
     media = MediaField(
         MP3StorageStyle('TMED'),
-        MP4StorageStyle("----:com.apple.iTunes:MEDIA"),
+        MP4StorageStyle('----:com.apple.iTunes:MEDIA'),
         StorageStyle('MEDIA'),
         ASFStorageStyle('WM/Media'),
     )
     albumdisambig = MediaField(
         # This tag mapping was invented for beets (not used by Picard, etc).
         MP3DescStorageStyle(u'MusicBrainz Album Comment'),
-        MP4StorageStyle("----:com.apple.iTunes:MusicBrainz Album Comment"),
+        MP4StorageStyle('----:com.apple.iTunes:MusicBrainz Album Comment'),
         StorageStyle('MUSICBRAINZ_ALBUMCOMMENT'),
         ASFStorageStyle('MusicBrainz/Album Comment'),
     )
@@ -1600,7 +1745,7 @@ class MediaFile(object):
     # Release date.
     date = DateField(
         MP3StorageStyle('TDRC'),
-        MP4StorageStyle("\xa9day"),
+        MP4StorageStyle('\xa9day'),
         StorageStyle('DATE'),
         ASFStorageStyle('WM/Year'),
         year=(StorageStyle('YEAR'),))
@@ -1623,13 +1768,13 @@ class MediaFile(object):
     # Nonstandard metadata.
     artist_credit = MediaField(
         MP3DescStorageStyle(u'Artist Credit'),
-        MP4StorageStyle("----:com.apple.iTunes:Artist Credit"),
+        MP4StorageStyle('----:com.apple.iTunes:Artist Credit'),
         StorageStyle('ARTIST_CREDIT'),
         ASFStorageStyle('beets/Artist Credit'),
     )
     albumartist_credit = MediaField(
         MP3DescStorageStyle(u'Album Artist Credit'),
-        MP4StorageStyle("----:com.apple.iTunes:Album Artist Credit"),
+        MP4StorageStyle('----:com.apple.iTunes:Album Artist Credit'),
         StorageStyle('ALBUMARTIST_CREDIT'),
         ASFStorageStyle('beets/Album Artist Credit'),
     )
@@ -1703,7 +1848,7 @@ class MediaFile(object):
         ),
         MP4StorageStyle(
             '----:com.apple.iTunes:replaygain_track_gain',
-            float_places=2, suffix=b' dB'
+            float_places=2, suffix=' dB'
         ),
         MP4SoundCheckStorageStyle(
             '----:com.apple.iTunes:iTunNORM',
