@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2013, Adrian Sampson.
+# Copyright 2016, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -13,25 +14,23 @@
 # included in all copies or substantial portions of the Software.
 
 """Some common functionality for beets' test cases."""
+from __future__ import division, absolute_import, print_function
+
 import time
 import sys
 import os
-import logging
 import tempfile
 import shutil
+import unittest
 from contextlib import contextmanager
 
-# Use unittest2 on Python < 2.7.
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
 
 # Mangle the search path to include the beets sources.
-sys.path.insert(0, '..')
+sys.path.insert(0, '..')  # noqa
 import beets.library
-from beets import importer
+from beets import importer, logging
 from beets.ui import commands
+from beets import util
 import beets
 
 # Make sure the development versions of the plugins are used
@@ -41,7 +40,8 @@ beetsplug.__path__ = [os.path.abspath(
 )]
 
 # Test resources path.
-RSRC = os.path.join(os.path.dirname(__file__), 'rsrc')
+RSRC = util.bytestring_path(os.path.join(os.path.dirname(__file__), 'rsrc'))
+PLUGINPATH = os.path.join(os.path.dirname(__file__), 'rsrc', 'beetsplug')
 
 # Propagate to root loger so nosetest can capture it
 log = logging.getLogger('beets')
@@ -51,6 +51,9 @@ log.setLevel(logging.DEBUG)
 # Dummy item creation.
 _item_ident = 0
 
+# OS feature test.
+HAVE_SYMLINK = hasattr(os, 'symlink')
+
 
 def item(lib=None):
     global _item_ident
@@ -74,7 +77,7 @@ def item(lib=None):
         comments=u'the comments',
         bpm=8,
         comp=True,
-        path='somepath' + str(_item_ident),
+        path='somepath{0}'.format(_item_ident),
         length=60.0,
         bitrate=128000,
         format='FLAC',
@@ -96,11 +99,11 @@ def album(lib=None):
     _item_ident += 1
     i = beets.library.Album(
         artpath=None,
-        albumartist='some album artist',
-        albumartist_sort='some sort album artist',
-        albumartist_credit='some album artist credit',
-        album='the album',
-        genre='the genre',
+        albumartist=u'some album artist',
+        albumartist_sort=u'some sort album artist',
+        albumartist_credit=u'some album artist credit',
+        album=u'the album',
+        genre=u'the genre',
         year=2014,
         month=2,
         day=5,
@@ -116,14 +119,35 @@ def album(lib=None):
 
 
 # Dummy import session.
-def import_session(lib=None, logfile=None, paths=[], query=[], cli=False):
+def import_session(lib=None, loghandler=None, paths=[], query=[], cli=False):
     cls = commands.TerminalImportSession if cli else importer.ImportSession
-    return cls(lib, logfile, paths, query)
+    return cls(lib, loghandler, paths, query)
+
+
+class Assertions(object):
+    """A mixin with additional unit test assertions."""
+
+    def assertExists(self, path):  # noqa
+        self.assertTrue(os.path.exists(util.syspath(path)),
+                        u'file does not exist: {!r}'.format(path))
+
+    def assertNotExists(self, path):  # noqa
+        self.assertFalse(os.path.exists(util.syspath(path)),
+                         u'file exists: {!r}'.format((path)))
+
+    def assert_equal_path(self, a, b):
+        """Check that two paths are equal."""
+        # The common case.
+        if a == b:
+            return
+
+        self.assertEqual(util.normpath(a), util.normpath(b),
+                         u'paths are not equal: {!r} and {!r}'.format(a, b))
 
 
 # A test harness for all beets tests.
 # Provides temporary, isolated configuration.
-class TestCase(unittest.TestCase):
+class TestCase(unittest.TestCase, Assertions):
     """A unittest.TestCase subclass that saves and restores beets'
     global configuration. This allows tests to make temporary
     modifications that will then be automatically removed when the test
@@ -162,14 +186,6 @@ class TestCase(unittest.TestCase):
         beets.config.clear()
         beets.config._materialized = False
 
-    def assertExists(self, path):
-        self.assertTrue(os.path.exists(path),
-                        'file does not exist: %s' % path)
-
-    def assertNotExists(self, path):
-        self.assertFalse(os.path.exists(path),
-                         'file exists: %s' % path)
-
 
 class LibTestCase(TestCase):
     """A test case that includes an in-memory library object (`lib`) and
@@ -222,7 +238,7 @@ class InputException(Exception):
     def __str__(self):
         msg = "Attempt to read with no input provided."
         if self.output is not None:
-            msg += " Output: %s" % self.output
+            msg += " Output: {!r}".format(self.output)
         return msg
 
 
@@ -236,7 +252,7 @@ class DummyOut(object):
         self.buf.append(s)
 
     def get(self):
-        return ''.join(self.buf)
+        return b''.join(self.buf)
 
     def clear(self):
         self.buf = []
@@ -251,7 +267,7 @@ class DummyIn(object):
         self.out = out
 
     def add(self, s):
-        self.buf.append(s + '\n')
+        self.buf.append(s + b'\n')
 
     def readline(self):
         if not self.buf:
@@ -340,3 +356,11 @@ def system_mock(name):
         yield
     finally:
         platform.system = old_system
+
+
+def slow_test(unused=None):
+    def _id(obj):
+        return obj
+    if 'SKIP_SLOW_TESTS' in os.environ:
+        return unittest.skip(u'test is slow')
+    return _id
