@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2014, Thomas Scholtes.
+# Copyright 2016, Thomas Scholtes.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -15,7 +16,7 @@
 """This module includes various helpers that provide fixtures, capture
 information or mock the environment.
 
-- The `control_stdin` and `capture_output` context managers allow one to
+- The `control_stdin` and `capture_stdout` context managers allow one to
   interact with the user interface.
 
 - `has_program` checks the presence of a command on the system.
@@ -30,6 +31,8 @@ information or mock the environment.
 """
 
 
+from __future__ import division, absolute_import, print_function
+
 import sys
 import os
 import os.path
@@ -41,15 +44,39 @@ from StringIO import StringIO
 from enum import Enum
 
 import beets
+from beets import logging
 from beets import config
 import beets.plugins
 from beets.library import Library, Item, Album
 from beets import importer
 from beets.autotag.hooks import AlbumInfo, TrackInfo
-from beets.mediafile import MediaFile
+from beets.mediafile import MediaFile, Image
+from beets.ui import _arg_encoding
+from beets import util
 
 # TODO Move AutotagMock here
-import _common
+from test import _common
+
+
+class LogCapture(logging.Handler):
+
+    def __init__(self):
+        logging.Handler.__init__(self)
+        self.messages = []
+
+    def emit(self, record):
+        self.messages.append(unicode(record.msg))
+
+
+@contextmanager
+def capture_log(logger='beets'):
+    capture = LogCapture()
+    log = logging.getLogger(logger)
+    log.addHandler(capture)
+    try:
+        yield capture.messages
+    finally:
+        log.removeHandler(capture)
 
 
 @contextmanager
@@ -92,12 +119,18 @@ def capture_stdout():
 def has_program(cmd, args=['--version']):
     """Returns `True` if `cmd` can be executed.
     """
+    full_cmd = [cmd] + args
+    for i, elem in enumerate(full_cmd):
+        if isinstance(elem, unicode):
+            full_cmd[i] = elem.encode(_arg_encoding())
     try:
         with open(os.devnull, 'wb') as devnull:
-            subprocess.check_call([cmd] + args, stderr=devnull,
+            subprocess.check_call(full_cmd, stderr=devnull,
                                   stdout=devnull, stdin=devnull)
     except OSError:
         return False
+    except subprocess.CalledProcessError:
+        return False
     else:
         return True
 
@@ -140,11 +173,11 @@ class TestHelper(object):
         self.config.read()
 
         self.config['plugins'] = []
-        self.config['verbose'] = True
-        self.config['color'] = False
+        self.config['verbose'] = 1
+        self.config['ui']['color'] = False
         self.config['threaded'] = False
 
-        self.libdir = os.path.join(self.temp_dir, 'libdir')
+        self.libdir = os.path.join(self.temp_dir, b'libdir')
         os.mkdir(self.libdir)
         self.config['directory'] = self.libdir
 
@@ -155,7 +188,7 @@ class TestHelper(object):
         self.lib = Library(dbpath, self.libdir)
 
     def teardown_beets(self):
-        del self.lib._connections
+        self.lib._close()
         if 'BEETSDIR' in os.environ:
             del os.environ['BEETSDIR']
         self.remove_temp_dir()
@@ -172,20 +205,21 @@ class TestHelper(object):
         beets.config['plugins'] = plugins
         beets.plugins.load_plugins(plugins)
         beets.plugins.find_plugins()
-        Item._types = beets.plugins.types(Item)
-        Album._types = beets.plugins.types(Album)
+        # Take a backup of the original _types to restore when unloading
+        Item._original_types = dict(Item._types)
+        Album._original_types = dict(Album._types)
+        Item._types.update(beets.plugins.types(Item))
+        Album._types.update(beets.plugins.types(Album))
 
     def unload_plugins(self):
         """Unload all plugins and remove the from the configuration.
         """
         # FIXME this should eventually be handled by a plugin manager
         beets.config['plugins'] = []
-        for plugin in beets.plugins._classes:
-            plugin.listeners = None
         beets.plugins._classes = set()
         beets.plugins._instances = {}
-        Item._types = {}
-        Album._types = {}
+        Item._types = Item._original_types
+        Album._types = Album._original_types
 
     def create_importer(self, item_count=1, album_count=1):
         """Create files to import and return corresponding session.
@@ -193,13 +227,13 @@ class TestHelper(object):
         Copies the specified number of files to a subdirectory of
         `self.temp_dir` and creates a `TestImportSession` for this path.
         """
-        import_dir = os.path.join(self.temp_dir, 'import')
+        import_dir = os.path.join(self.temp_dir, b'import')
         if not os.path.isdir(import_dir):
             os.mkdir(import_dir)
 
         album_no = 0
         while album_count:
-            album = u'album {0}'.format(album_no)
+            album = util.bytestring_path(u'album {0}'.format(album_no))
             album_dir = os.path.join(import_dir, album)
             if os.path.exists(album_dir):
                 album_no += 1
@@ -210,9 +244,10 @@ class TestHelper(object):
             track_no = 0
             album_item_count = item_count
             while album_item_count:
-                title = 'track {0}'.format(track_no)
-                src = os.path.join(_common.RSRC, 'full.mp3')
-                dest = os.path.join(album_dir, '{0}.mp3'.format(title))
+                title = u'track {0}'.format(track_no)
+                src = os.path.join(_common.RSRC, b'full.mp3')
+                title_file = util.bytestring_path('{0}.mp3'.format(title))
+                dest = os.path.join(album_dir, title_file)
                 if os.path.exists(dest):
                     track_no += 1
                     continue
@@ -233,7 +268,7 @@ class TestHelper(object):
         config['import']['autotag'] = False
         config['import']['resume'] = False
 
-        return TestImportSession(self.lib, logfile=None, query=None,
+        return TestImportSession(self.lib, loghandler=None, query=None,
                                  paths=[import_dir])
 
     # Library fixtures methods
@@ -273,11 +308,19 @@ class TestHelper(object):
 
         If `path` is not set in `values` it is set to `item.destination()`.
         """
+        # When specifying a path, store it normalized (as beets does
+        # ordinarily).
+        if 'path' in values:
+            values['path'] = util.normpath(values['path'])
+
         item = self.create_item(**values)
         item.add(self.lib)
+
+        # Ensure every item has a path.
         if 'path' not in values:
             item['path'] = item.destination()
             item.store()
+
         return item
 
     def add_item_fixture(self, **values):
@@ -285,7 +328,8 @@ class TestHelper(object):
         """
         item = self.create_item(**values)
         extension = item['format'].lower()
-        item['path'] = os.path.join(_common.RSRC, 'min.' + extension)
+        item['path'] = os.path.join(_common.RSRC,
+                                    util.bytestring_path('min.' + extension))
         item.add(self.lib)
         item.move(copy=True)
         item.store()
@@ -300,9 +344,9 @@ class TestHelper(object):
         """
         # TODO base this on `add_item()`
         items = []
-        path = os.path.join(_common.RSRC, 'full.' + ext)
+        path = os.path.join(_common.RSRC, util.bytestring_path('full.' + ext))
         for i in range(count):
-            item = Item.from_path(str(path))
+            item = Item.from_path(path)
             item.album = u'\u00e4lbum {0}'.format(i)  # Check unicode paths
             item.title = u't\u00eftle {0}'.format(i)
             item.add(self.lib)
@@ -315,9 +359,9 @@ class TestHelper(object):
         """Add an album with files to the database.
         """
         items = []
-        path = os.path.join(_common.RSRC, 'full.' + ext)
+        path = os.path.join(_common.RSRC, util.bytestring_path('full.' + ext))
         for i in range(track_count):
-            item = Item.from_path(str(path))
+            item = Item.from_path(bytes(path))
             item.album = u'\u00e4lbum'  # Check unicode paths
             item.title = u't\u00eftle {0}'.format(i)
             item.add(self.lib)
@@ -326,18 +370,33 @@ class TestHelper(object):
             items.append(item)
         return self.lib.add_album(items)
 
-    def create_mediafile_fixture(self, ext='mp3'):
+    def create_mediafile_fixture(self, ext='mp3', images=[]):
         """Copies a fixture mediafile with the extension to a temporary
         location and returns the path.
 
         It keeps track of the created locations and will delete the with
         `remove_mediafile_fixtures()`
+
+        `images` is a subset of 'png', 'jpg', and 'tiff'. For each
+        specified extension a cover art image is added to the media
+        file.
         """
-        src = os.path.join(_common.RSRC, 'full.' + ext)
+        src = os.path.join(_common.RSRC, util.bytestring_path('full.' + ext))
         handle, path = mkstemp()
         os.close(handle)
         shutil.copyfile(src, path)
 
+        if images:
+            mediafile = MediaFile(path)
+            imgs = []
+            for img_ext in images:
+                file = util.bytestring_path('image-2x3.{0}'.format(img_ext))
+                img_path = os.path.join(_common.RSRC, file)
+                with open(img_path, 'rb') as f:
+                    imgs.append(Image(f.read()))
+            mediafile.images = imgs
+            mediafile.save()
+
         if not hasattr(self, '_mediafile_fixtures'):
             self._mediafile_fixtures = []
         self._mediafile_fixtures.append(path)
@@ -367,7 +426,7 @@ class TestHelper(object):
     def run_with_output(self, *args):
         with capture_stdout() as out:
             self.run_command(*args)
-        return out.getvalue()
+        return out.getvalue().decode('utf-8')
 
     # Safe file operations
 
@@ -397,9 +456,9 @@ class TestHelper(object):
 
         parent = os.path.dirname(path)
         if not os.path.isdir(parent):
-            os.makedirs(parent)
+            os.makedirs(util.syspath(parent))
 
-        with open(path, 'a+') as f:
+        with open(util.syspath(path), 'a+') as f:
             f.write(content)
         return path
 
@@ -478,14 +537,14 @@ def generate_album_info(album_id, track_ids):
     """
     tracks = [generate_track_info(id) for id in track_ids]
     album = AlbumInfo(
-        album_id='album info',
-        album='album info',
-        artist='album info',
-        artist_id='album info',
+        album_id=u'album info',
+        album=u'album info',
+        artist=u'album info',
+        artist_id=u'album info',
         tracks=tracks,
     )
     for field in ALBUM_INFO_FIELDS:
-        setattr(album, field, 'album info')
+        setattr(album, field, u'album info')
 
     return album
 
@@ -504,11 +563,11 @@ def generate_track_info(track_id='track info', values={}):
     string fields are set to "track info".
     """
     track = TrackInfo(
-        title='track info',
+        title=u'track info',
         track_id=track_id,
     )
     for field in TRACK_INFO_FIELDS:
-        setattr(track, field, 'track info')
+        setattr(track, field, u'track info')
     for field, value in values.items():
         setattr(track, field, value)
     return track
