@@ -1,12 +1,16 @@
+# -*- coding: utf-8 -*-
+
+from __future__ import division, absolute_import, print_function
+
 import os
 import os.path
 import tempfile
 import shutil
 
-from _common import unittest
+from test._common import unittest
 from beets import config
 from beets.library import Item, Album, Library
-from beetsplug.importfeeds import album_imported, ImportFeedsPlugin
+from beetsplug.importfeeds import ImportFeedsPlugin
 
 
 class ImportfeedsTestTest(unittest.TestCase):
@@ -30,13 +34,30 @@ class ImportfeedsTestTest(unittest.TestCase):
         self.lib.add(album)
         self.lib.add(item)
 
-        album_imported(self.lib, album)
+        self.importfeeds.album_imported(self.lib, album)
         playlist_path = os.path.join(self.feeds_dir,
                                      os.listdir(self.feeds_dir)[0])
         self.assertTrue(playlist_path.endswith('album_name.m3u'))
         with open(playlist_path) as playlist:
             self.assertIn(item_path, playlist.read())
 
+    def test_playlist_in_subdir(self):
+        config['importfeeds']['formats'] = 'm3u'
+        config['importfeeds']['m3u_name'] = \
+            os.path.join('subdir', 'imported.m3u')
+        album = Album(album='album/name', id=1)
+        item_path = os.path.join('path', 'to', 'item')
+        item = Item(title='song', album_id=1, path=item_path)
+        self.lib.add(album)
+        self.lib.add(item)
+
+        self.importfeeds.album_imported(self.lib, album)
+        playlist = os.path.join(self.feeds_dir,
+                                config['importfeeds']['m3u_name'].get())
+        playlist_subdir = os.path.dirname(playlist)
+        self.assertTrue(os.path.isdir(playlist_subdir))
+        self.assertTrue(os.path.isfile(playlist))
+
 
 def suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
