@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2013, Adrian Sampson.
+# Copyright 2016, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -15,19 +16,22 @@
 """Automatically-generated blanket testing for the MediaFile metadata
 layer.
 """
+from __future__ import division, absolute_import, print_function
+
 import os
 import shutil
 import tempfile
 import datetime
 import time
 
-import _common
-from _common import unittest
+from test import _common
+from test._common import unittest
 from beets.mediafile import MediaFile, MediaField, Image, \
     MP3DescStorageStyle, StorageStyle, MP4StorageStyle, \
-    ASFStorageStyle, ImageType
+    ASFStorageStyle, ImageType, CoverArtField
 from beets.library import Item
 from beets.plugins import BeetsPlugin
+from beets.util import bytestring_path
 
 
 class ArtTestMixin(object):
@@ -37,7 +41,8 @@ class ArtTestMixin(object):
     @property
     def png_data(self):
         if not self._png_data:
-            with open(os.path.join(_common.RSRC, 'image-2x3.png'), 'rb') as f:
+            image_file = os.path.join(_common.RSRC, b'image-2x3.png')
+            with open(image_file, 'rb') as f:
                 self._png_data = f.read()
         return self._png_data
     _png_data = None
@@ -45,7 +50,8 @@ class ArtTestMixin(object):
     @property
     def jpg_data(self):
         if not self._jpg_data:
-            with open(os.path.join(_common.RSRC, 'image-2x3.jpg'), 'rb') as f:
+            image_file = os.path.join(_common.RSRC, b'image-2x3.jpg')
+            with open(image_file, 'rb') as f:
                 self._jpg_data = f.read()
         return self._jpg_data
     _jpg_data = None
@@ -53,7 +59,8 @@ class ArtTestMixin(object):
     @property
     def tiff_data(self):
         if not self._jpg_data:
-            with open(os.path.join(_common.RSRC, 'image-2x3.tiff'), 'rb') as f:
+            image_file = os.path.join(_common.RSRC, b'image-2x3.tiff')
+            with open(image_file, 'rb') as f:
                 self._jpg_data = f.read()
         return self._jpg_data
     _jpg_data = None
@@ -74,6 +81,20 @@ class ArtTestMixin(object):
         mediafile = MediaFile(mediafile.path)
         self.assertEqual(mediafile.art, self.jpg_data)
 
+    def test_delete_art(self):
+        mediafile = self._mediafile_fixture('empty')
+        mediafile.art = self.jpg_data
+        mediafile.save()
+
+        mediafile = MediaFile(mediafile.path)
+        self.assertIsNotNone(mediafile.art)
+
+        del mediafile.art
+        mediafile.save()
+
+        mediafile = MediaFile(mediafile.path)
+        self.assertIsNone(mediafile.art)
+
 
 class ImageStructureTestMixin(ArtTestMixin):
     """Test reading and writing multiple image tags.
@@ -91,18 +112,18 @@ class ImageStructureTestMixin(ArtTestMixin):
         image = next(i for i in mediafile.images
                      if i.mime_type == 'image/png')
         self.assertEqual(image.data, self.png_data)
-        self.assertExtendedImageAttributes(image, desc='album cover',
+        self.assertExtendedImageAttributes(image, desc=u'album cover',
                                            type=ImageType.front)
 
         image = next(i for i in mediafile.images
                      if i.mime_type == 'image/jpeg')
         self.assertEqual(image.data, self.jpg_data)
-        self.assertExtendedImageAttributes(image, desc='the artist',
+        self.assertExtendedImageAttributes(image, desc=u'the artist',
                                            type=ImageType.artist)
 
     def test_set_image_structure(self):
         mediafile = self._mediafile_fixture('empty')
-        image = Image(data=self.png_data, desc='album cover',
+        image = Image(data=self.png_data, desc=u'album cover',
                       type=ImageType.front)
         mediafile.images = [image]
         mediafile.save()
@@ -113,14 +134,14 @@ class ImageStructureTestMixin(ArtTestMixin):
         image = mediafile.images[0]
         self.assertEqual(image.data, self.png_data)
         self.assertEqual(image.mime_type, 'image/png')
-        self.assertExtendedImageAttributes(image, desc='album cover',
+        self.assertExtendedImageAttributes(image, desc=u'album cover',
                                            type=ImageType.front)
 
     def test_add_image_structure(self):
         mediafile = self._mediafile_fixture('image')
         self.assertEqual(len(mediafile.images), 2)
 
-        image = Image(data=self.png_data, desc='the composer',
+        image = Image(data=self.png_data, desc=u'the composer',
                       type=ImageType.composer)
         mediafile.images += [image]
         mediafile.save()
@@ -128,13 +149,30 @@ class ImageStructureTestMixin(ArtTestMixin):
         mediafile = MediaFile(mediafile.path)
         self.assertEqual(len(mediafile.images), 3)
 
-        images = (i for i in mediafile.images if i.desc == 'the composer')
+        images = (i for i in mediafile.images if i.desc == u'the composer')
         image = next(images, None)
         self.assertExtendedImageAttributes(
-            image, desc='the composer', type=ImageType.composer
+            image, desc=u'the composer', type=ImageType.composer
         )
 
-    def assertExtendedImageAttributes(self, image, **kwargs):
+    def test_delete_image_structures(self):
+        mediafile = self._mediafile_fixture('image')
+        self.assertEqual(len(mediafile.images), 2)
+
+        del mediafile.images
+        mediafile.save()
+
+        mediafile = MediaFile(mediafile.path)
+        self.assertEqual(len(mediafile.images), 0)
+
+    def test_guess_cover(self):
+        mediafile = self._mediafile_fixture('image')
+        self.assertEqual(len(mediafile.images), 2)
+        cover = CoverArtField.guess_cover_image(mediafile.images)
+        self.assertEqual(cover.desc, u'album cover')
+        self.assertEqual(mediafile.art, cover.data)
+
+    def assertExtendedImageAttributes(self, image, **kwargs):  # noqa
         """Ignore extended image attributes in the base tests.
         """
         pass
@@ -143,7 +181,7 @@ class ImageStructureTestMixin(ArtTestMixin):
 class ExtendedImageStructureTestMixin(ImageStructureTestMixin):
     """Checks for additional attributes in the image structure."""
 
-    def assertExtendedImageAttributes(self, image, desc=None, type=None):
+    def assertExtendedImageAttributes(self, image, desc=None, type=None):  # noqa
         self.assertEqual(image.desc, desc)
         self.assertEqual(image.type, type)
 
@@ -151,7 +189,7 @@ class ExtendedImageStructureTestMixin(ImageStructureTestMixin):
         mediafile = self._mediafile_fixture('image')
         self.assertEqual(len(mediafile.images), 2)
 
-        image = Image(data=self.tiff_data, desc='the composer',
+        image = Image(data=self.tiff_data, desc=u'the composer',
                       type=ImageType.composer)
         mediafile.images += [image]
         mediafile.save()
@@ -160,17 +198,17 @@ class ExtendedImageStructureTestMixin(ImageStructureTestMixin):
         self.assertEqual(len(mediafile.images), 3)
 
         # WMA does not preserve the order, so we have to work around this
-        image = filter(lambda i: i.mime_type == 'image/tiff',
-                       mediafile.images)[0]
+        image = list(filter(lambda i: i.mime_type == 'image/tiff',
+                     mediafile.images))[0]
         self.assertExtendedImageAttributes(
-            image, desc='the composer', type=ImageType.composer)
+            image, desc=u'the composer', type=ImageType.composer)
 
 
 class LazySaveTestMixin(object):
     """Mediafile should only write changes when tags have changed
     """
 
-    @unittest.skip('not yet implemented')
+    @unittest.skip(u'not yet implemented')
     def test_unmodified(self):
         mediafile = self._mediafile_fixture('full')
         mtime = self._set_past_mtime(mediafile.path)
@@ -179,7 +217,7 @@ class LazySaveTestMixin(object):
         mediafile.save()
         self.assertEqual(os.stat(mediafile.path).st_mtime, mtime)
 
-    @unittest.skip('not yet implemented')
+    @unittest.skip(u'not yet implemented')
     def test_same_tag_value(self):
         mediafile = self._mediafile_fixture('full')
         mtime = self._set_past_mtime(mediafile.path)
@@ -198,14 +236,14 @@ class LazySaveTestMixin(object):
         mediafile.save()
         self.assertEqual(os.stat(mediafile.path).st_mtime, mtime)
 
-    @unittest.skip('not yet implemented')
+    @unittest.skip(u'not yet implemented')
     def test_tag_value_change(self):
         mediafile = self._mediafile_fixture('full')
         mtime = self._set_past_mtime(mediafile.path)
         self.assertEqual(os.stat(mediafile.path).st_mtime, mtime)
 
         mediafile.title = mediafile.title
-        mediafile.album = 'another'
+        mediafile.album = u'another'
         mediafile.save()
         self.assertNotEqual(os.stat(mediafile.path).st_mtime, mtime)
 
@@ -214,7 +252,7 @@ class LazySaveTestMixin(object):
         mtime = self._set_past_mtime(mediafile.path)
         self.assertEqual(os.stat(mediafile.path).st_mtime, mtime)
 
-        mediafile.update({'title': mediafile.title, 'album': 'another'})
+        mediafile.update({'title': mediafile.title, 'album': u'another'})
         mediafile.save()
         self.assertNotEqual(os.stat(mediafile.path).st_mtime, mtime)
 
@@ -238,7 +276,7 @@ class GenreListTestMixin(object):
         mediafile.save()
 
         mediafile = MediaFile(mediafile.path)
-        self.assertItemsEqual(mediafile.genres, ['one', 'two'])
+        self.assertItemsEqual(mediafile.genres, [u'one', u'two'])
 
     def test_write_genre_list_get_first(self):
         mediafile = self._mediafile_fixture('empty')
@@ -246,11 +284,11 @@ class GenreListTestMixin(object):
         mediafile.save()
 
         mediafile = MediaFile(mediafile.path)
-        self.assertEqual(mediafile.genre, 'one')
+        self.assertEqual(mediafile.genre, u'one')
 
     def test_append_genre_list(self):
         mediafile = self._mediafile_fixture('full')
-        self.assertEqual(mediafile.genre, 'the genre')
+        self.assertEqual(mediafile.genre, u'the genre')
         mediafile.genres += [u'another']
         mediafile.save()
 
@@ -273,11 +311,11 @@ class ExtendedFieldTestMixin(object):
         plugin.add_media_field('customtag', field_extension)
 
         mediafile = self._mediafile_fixture('empty')
-        mediafile.customtag = 'F#'
+        mediafile.customtag = u'F#'
         mediafile.save()
 
         mediafile = MediaFile(mediafile.path)
-        self.assertEqual(mediafile.customtag, 'F#')
+        self.assertEqual(mediafile.customtag, u'F#')
         delattr(MediaFile, 'customtag')
         Item._media_fields.remove('customtag')
 
@@ -288,10 +326,10 @@ class ExtendedFieldTestMixin(object):
         mediafile = self._mediafile_fixture('empty')
         self.assertIsNone(mediafile.customtag)
 
-        item = Item(path=mediafile.path, customtag='Gb')
+        item = Item(path=mediafile.path, customtag=u'Gb')
         item.write()
         mediafile = MediaFile(mediafile.path)
-        self.assertEqual(mediafile.customtag, 'Gb')
+        self.assertEqual(mediafile.customtag, u'Gb')
 
         delattr(MediaFile, 'customtag')
         Item._media_fields.remove('customtag')
@@ -301,11 +339,11 @@ class ExtendedFieldTestMixin(object):
         plugin.add_media_field('customtag', field_extension)
 
         mediafile = self._mediafile_fixture('empty')
-        mediafile.update({'customtag': 'F#'})
+        mediafile.update({'customtag': u'F#'})
         mediafile.save()
 
         item = Item.from_path(mediafile.path)
-        self.assertEqual(item['customtag'], 'F#')
+        self.assertEqual(item['customtag'], u'F#')
 
         delattr(MediaFile, 'customtag')
         Item._media_fields.remove('customtag')
@@ -313,12 +351,14 @@ class ExtendedFieldTestMixin(object):
     def test_invalid_descriptor(self):
         with self.assertRaises(ValueError) as cm:
             MediaFile.add_field('somekey', True)
-        self.assertIn('must be an instance of MediaField', str(cm.exception))
+        self.assertIn(u'must be an instance of MediaField',
+                      unicode(cm.exception))
 
     def test_overwrite_property(self):
         with self.assertRaises(ValueError) as cm:
             MediaFile.add_field('artist', MediaField())
-        self.assertIn('property "artist" already exists', str(cm.exception))
+        self.assertIn(u'property "artist" already exists',
+                      unicode(cm.exception))
 
 
 class ReadWriteTestBase(ArtTestMixin, GenreListTestMixin,
@@ -624,22 +664,22 @@ class ReadWriteTestBase(ArtTestMixin, GenreListTestMixin,
         self.assertIsNone(mediafile.date)
         self.assertIsNone(mediafile.year)
 
-    def assertTags(self, mediafile, tags):
+    def assertTags(self, mediafile, tags):  # noqa
         errors = []
         for key, value in tags.items():
             try:
                 value2 = getattr(mediafile, key)
             except AttributeError:
-                errors.append('Tag %s does not exist' % key)
+                errors.append(u'Tag %s does not exist' % key)
             else:
                 if value2 != value:
-                    errors.append('Tag %s: %s != %s' % (key, value2, value))
+                    errors.append(u'Tag %s: %r != %r' % (key, value2, value))
         if any(errors):
-            errors = ['Tags did not match'] + errors
+            errors = [u'Tags did not match'] + errors
             self.fail('\n  '.join(errors))
 
     def _mediafile_fixture(self, name):
-        name = name + '.' + self.extension
+        name = bytestring_path(name + '.' + self.extension)
         src = os.path.join(_common.RSRC, name)
         target = os.path.join(self.temp_dir, name)
         shutil.copy(src, target)
@@ -674,6 +714,7 @@ class ReadWriteTestBase(ArtTestMixin, GenreListTestMixin,
         tags['original_year'] = original_date.year
         tags['original_month'] = original_date.month
         tags['original_day'] = original_date.day
+
         return tags
 
 
@@ -706,6 +747,10 @@ class MP3Test(ReadWriteTestBase, PartialTestMixin,
         'channels': 1,
     }
 
+    def test_unknown_apic_type(self):
+        mediafile = self._mediafile_fixture('image_unknown_type')
+        self.assertEqual(mediafile.images[0].type, ImageType.other)
+
 
 class MP4Test(ReadWriteTestBase, PartialTestMixin,
               ImageStructureTestMixin, unittest.TestCase):
@@ -724,16 +769,20 @@ class MP4Test(ReadWriteTestBase, PartialTestMixin,
         with self.assertRaises(ValueError):
             mediafile.images = [Image(data=self.tiff_data)]
 
+    def test_guess_cover(self):
+        # There is no metadata associated with images, we pick one at random
+        pass
+
 
 class AlacTest(ReadWriteTestBase, unittest.TestCase):
     extension = 'alac.m4a'
     audio_properties = {
         'length': 1.0,
-        'bitrate': 55072,
-        'format': 'ALAC',
-        'samplerate': 0,
-        'bitdepth': 0,
-        'channels': 0,
+        'bitrate': 21830,
+        # 'format': 'ALAC',
+        'samplerate': 44100,
+        'bitdepth': 16,
+        'channels': 1,
     }
 
 
@@ -742,7 +791,7 @@ class MusepackTest(ReadWriteTestBase, unittest.TestCase):
     audio_properties = {
         'length': 1.0,
         'bitrate': 23458,
-        'format': 'Musepack',
+        'format': u'Musepack',
         'samplerate': 44100,
         'bitdepth': 0,
         'channels': 2,
@@ -755,7 +804,7 @@ class WMATest(ReadWriteTestBase, ExtendedImageStructureTestMixin,
     audio_properties = {
         'length': 1.0,
         'bitrate': 128000,
-        'format': 'Windows Media',
+        'format': u'Windows Media',
         'samplerate': 44100,
         'bitdepth': 0,
         'channels': 1,
@@ -770,6 +819,12 @@ class WMATest(ReadWriteTestBase, ExtendedImageStructureTestMixin,
         mediafile = MediaFile(mediafile.path)
         self.assertIn(mediafile.genre, [u'one', u'two'])
 
+    def test_read_pure_tags(self):
+        mediafile = self._mediafile_fixture('pure')
+        self.assertEqual(mediafile.comments, u'the comments')
+        self.assertEqual(mediafile.title, u'the title')
+        self.assertEqual(mediafile.artist, u'the artist')
+
 
 class OggTest(ReadWriteTestBase, ExtendedImageStructureTestMixin,
               unittest.TestCase):
@@ -777,7 +832,7 @@ class OggTest(ReadWriteTestBase, ExtendedImageStructureTestMixin,
     audio_properties = {
         'length': 1.0,
         'bitrate': 48000,
-        'format': 'OGG',
+        'format': u'OGG',
         'samplerate': 44100,
         'bitdepth': 0,
         'channels': 1,
@@ -807,6 +862,12 @@ class OggTest(ReadWriteTestBase, ExtendedImageStructureTestMixin,
         mediafile = MediaFile(mediafile.path)
         self.assertFalse('coverart' in mediafile.mgfile)
 
+    def test_date_tag_with_slashes(self):
+        mediafile = self._mediafile_fixture('date_with_slashes')
+        self.assertEqual(mediafile.year, 2005)
+        self.assertEqual(mediafile.month, 6)
+        self.assertEqual(mediafile.day, 5)
+
 
 class FlacTest(ReadWriteTestBase, PartialTestMixin,
                ExtendedImageStructureTestMixin,
@@ -815,19 +876,20 @@ class FlacTest(ReadWriteTestBase, PartialTestMixin,
     audio_properties = {
         'length': 1.0,
         'bitrate': 175120,
-        'format': 'FLAC',
+        'format': u'FLAC',
         'samplerate': 44100,
         'bitdepth': 16,
         'channels': 1,
     }
 
 
-class ApeTest(ReadWriteTestBase, unittest.TestCase):
+class ApeTest(ReadWriteTestBase, ExtendedImageStructureTestMixin,
+              unittest.TestCase):
     extension = 'ape'
     audio_properties = {
         'length': 1.0,
         'bitrate': 112040,
-        'format': 'APE',
+        'format': u'APE',
         'samplerate': 44100,
         'bitdepth': 16,
         'channels': 1,
@@ -839,7 +901,7 @@ class WavpackTest(ReadWriteTestBase, unittest.TestCase):
     audio_properties = {
         'length': 1.0,
         'bitrate': 108744,
-        'format': 'WavPack',
+        'format': u'WavPack',
         'samplerate': 44100,
         'bitdepth': 0,
         'channels': 1,
@@ -851,7 +913,7 @@ class OpusTest(ReadWriteTestBase, unittest.TestCase):
     audio_properties = {
         'length': 1.0,
         'bitrate': 57984,
-        'format': 'Opus',
+        'format': u'Opus',
         'samplerate': 48000,
         'bitdepth': 0,
         'channels': 1,
@@ -863,7 +925,7 @@ class AIFFTest(ReadWriteTestBase, unittest.TestCase):
     audio_properties = {
         'length': 1.0,
         'bitrate': 705600,
-        'format': 'AIFF',
+        'format': u'AIFF',
         'samplerate': 44100,
         'bitdepth': 0,
         'channels': 1,
@@ -873,13 +935,13 @@ class AIFFTest(ReadWriteTestBase, unittest.TestCase):
 class MediaFieldTest(unittest.TestCase):
 
     def test_properties_from_fields(self):
-        path = os.path.join(_common.RSRC, 'full.mp3')
+        path = os.path.join(_common.RSRC, b'full.mp3')
         mediafile = MediaFile(path)
         for field in MediaFile.fields():
             self.assertTrue(hasattr(mediafile, field))
 
     def test_properties_from_readable_fields(self):
-        path = os.path.join(_common.RSRC, 'full.mp3')
+        path = os.path.join(_common.RSRC, b'full.mp3')
         mediafile = MediaFile(path)
         for field in MediaFile.readable_fields():
             self.assertTrue(hasattr(mediafile, field))
