@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2013, Adrian Sampson.
+# Copyright 2016, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -14,7 +15,9 @@
 
 """Test the "pipeline.py" restricted parallel programming library.
 """
-from _common import unittest
+from __future__ import division, absolute_import, print_function
+
+from test._common import unittest
 from beets.util import pipeline
 
 
@@ -130,7 +133,7 @@ class ExceptionTest(unittest.TestCase):
         pl = pipeline.Pipeline((_produce(), _exc_work()))
         pull = pl.pull()
         for i in range(3):
-            pull.next()
+            next(pull)
         self.assertRaises(TestException, pull.next)
 
 
