@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2013, Adrian Sampson.
+# Copyright 2016, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -14,48 +15,50 @@
 
 """Tests for BPD and music playing.
 """
-from _common import unittest
+from __future__ import division, absolute_import, print_function
+
+from test._common import unittest
 from beetsplug import bpd
 
 
 class CommandParseTest(unittest.TestCase):
     def test_no_args(self):
-        s = ur'command'
+        s = r'command'
         c = bpd.Command(s)
         self.assertEqual(c.name, u'command')
         self.assertEqual(c.args, [])
 
     def test_one_unquoted_arg(self):
-        s = ur'command hello'
+        s = r'command hello'
         c = bpd.Command(s)
         self.assertEqual(c.name, u'command')
         self.assertEqual(c.args, [u'hello'])
 
     def test_two_unquoted_args(self):
-        s = ur'command hello there'
+        s = r'command hello there'
         c = bpd.Command(s)
         self.assertEqual(c.name, u'command')
         self.assertEqual(c.args, [u'hello', u'there'])
 
     def test_one_quoted_arg(self):
-        s = ur'command "hello there"'
+        s = r'command "hello there"'
         c = bpd.Command(s)
         self.assertEqual(c.name, u'command')
         self.assertEqual(c.args, [u'hello there'])
 
     def test_heterogenous_args(self):
-        s = ur'command "hello there" sir'
+        s = r'command "hello there" sir'
         c = bpd.Command(s)
         self.assertEqual(c.name, u'command')
         self.assertEqual(c.args, [u'hello there', u'sir'])
 
     def test_quote_in_arg(self):
-        s = ur'command "hello \" there"'
+        s = r'command "hello \" there"'
         c = bpd.Command(s)
         self.assertEqual(c.args, [u'hello " there'])
 
     def test_backslash_in_arg(self):
-        s = ur'command "hello \\ there"'
+        s = r'command "hello \\ there"'
         c = bpd.Command(s)
         self.assertEqual(c.args, [u'hello \ there'])
 
