@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2013, Adrian Sampson.
+# Copyright 2016, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -14,7 +15,11 @@
 
 """Tests for template engine.
 """
-from _common import unittest
+from __future__ import division, absolute_import, print_function
+
+import warnings
+
+from test._common import unittest
 from beets.util import functemplate
 
 
@@ -207,6 +212,13 @@ class ParseTest(unittest.TestCase):
         self._assert_call(arg_parts[0], u"bar", 1)
         self.assertEqual(list(_normexpr(arg_parts[0].args[0])), [u'baz'])
 
+    def test_fail_on_utf8(self):
+        parts = u'é'.encode('utf8')
+        warnings.simplefilter("ignore")
+        with self.assertRaises(UnicodeDecodeError):
+            functemplate._parse(parts)
+        warnings.simplefilter("default")
+
 
 class EvalTest(unittest.TestCase):
     def _eval(self, template):
